; isr.asm - This file is a part of NutsOS

; NutsOS
; Copyright (C) 2013  Free Software Foundation, Inc.

; NutsOS is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 3 of the License, or
; (at your option) any later version.

; NutsOS is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License along
; with NutsOS; If not, see <http://www.gnu.org/licenses/>.

; Valentin Verdier <valentin.verdier03@gmail.com>

[BITS 32]
[SECTION .text]

extern IsrDefault, IsrGPFault, IsrSegFault, IsrPageFault, IsrClock, IsrKeyboard, ExecSystemCall, PIC_Send_EOI
global _DefaultInterrupt, _SEG_Fault, _GP_Fault, _Page_Fault, _KeyboardInterrupt, _ClockInterrupt, _SystemCall

%macro REGS_SAVE 0
	pushad
	push ds
	push es
	push fs
	push gs
	push ebx
	mov bx, 0x10
	mov ds, bx
	pop ebx
%endmacro

%macro REGS_RESTORE 0
	pop gs
	pop fs
	pop es
	pop ds
	popad
%endmacro

_DefaultInterrupt:
	REGS_SAVE
	call IsrDefault
	call PIC_Send_EOI
	REGS_RESTORE
	iret

_SEG_Fault:
	REGS_SAVE
	call IsrSegFault
	REGS_RESTORE
	iret

_GP_Fault:
	REGS_SAVE
	call IsrGPFault
	REGS_RESTORE
	add esp, 4 ; Pour le décalage créé par le code d'erreur dans la pile
	iret

_Page_Fault:
	REGS_SAVE
	call IsrPageFault
	REGS_RESTORE
	add esp, 4 ; Pour le décalage créé par le code d'erreur dans la pile
	iret

_KeyboardInterrupt:
	REGS_SAVE
	call IsrKeyboard
	call PIC_Send_EOI
	REGS_RESTORE
	iret

_ClockInterrupt:
	REGS_SAVE
	call IsrClock
	call PIC_Send_EOI
	REGS_RESTORE
	iret

_SystemCall:
	REGS_SAVE
	push eax
	call ExecSystemCall
	pop eax
	REGS_RESTORE
	iret
