; asm_utils.asm - This file is a part of NutsOS

; NutsOS
; Copyright (C) 2013  Free Software Foundation, Inc.

; NutsOS is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 3 of the License, or
; (at your option) any later version.

; NutsOS is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License along
; with NutsOS; If not, see <http://www.gnu.org/licenses/>.

; Valentin Verdier <valentin.verdier03@gmail.com>

[BITS 32]
[SECTION .text]
global _ClearInterrupt, _StartInterrupt, _EnablePaging, _LoadGdtR, _LoadIdtR, _LoadTaskRegister, _EmptyTLB, _LoadCr3, _ReadCr3, _ReadCr2, _SaveEflags, _RestoreEflags, _GetStackFrameAddr, _AtomicTestAndSet

_ClearInterrupt:
	cli
	ret

_StartInterrupt:
	sti
	ret

_EnablePaging:
	push eax
	mov eax, 0x303000
	mov cr3, eax
	mov eax, cr0
	or eax, 0x80000000
	mov cr0, eax
	pop eax
	ret

_LoadGdtR:
	mov eax, [esp+4]
	lgdt [eax]
	mov ax, 0x10
	mov ds, ax
	mov es, ax
	mov fs, ax
	mov gs, ax
	mov ax, 0x18
	mov ss, ax
	jmp 0x08:.next
	.next:
	ret

_LoadIdtR:
	push eax
	mov eax, [esp+8]
	lidt [eax]
	pop eax
	ret

_LoadTaskRegister:
	push ax
	mov ax, 0x38
	ltr ax
	pop ax
	ret

_EmptyTLB:
	push eax
	mov eax, cr3
	mov cr3, eax
	pop eax
	ret

_LoadCr3:
	push eax
	mov eax, [esp+8]
	mov cr3, eax
	pop eax
	ret

_ReadCr3:
	mov eax, cr3
	ret

_ReadCr2:
	mov eax, cr2
	ret

_SaveEflags:
	pushfd
	pop eax
	ret

_RestoreEflags:
	push eax
	mov eax, [esp+8]
	push eax
	popfd
	pop eax
	ret

_AtomicTestAndSet:
	push ebx
	push edx
	xor eax, eax
	mov edx, 1
	mov ebx, [esp + 12]
	lock cmpxchg [ebx], edx
	pop edx
	pop ebx
	ret

_GetStackFrameAddr:
	push ebx
	push ecx
	mov ecx, [esp + 12]
	mov ebx, ebp
.loopTop:
	cmp ecx, 0
	je .loopEnd
	mov eax, [ebx]
	mov ebx, eax
	dec ecx
	jmp .loopTop
.loopEnd:
	mov eax, ebx
	pop ecx
	pop ebx
	ret
